--[[
    This class lets the master looter appoint "pack mules"
    The pack mules (player) will automatically receive all items
    that their eligible for according to the rules as set up by the ML
]]
local _, GL = ...;

GL.PackMuleUI = GL.PackMuleUI or {
    setupWindowIsActive = false,

    UIComponents = {
        Input = {
            SpecificItemRules = {},
        },
    },
};

local AceGUI = GL.AceGUI;
local Settings = GL.Settings;
local PackMuleUI = GL.PackMuleUI;

function PackMuleUI:drawSetupWindow()
    GL:debug("PackMuleUI:drawSetupWindow");

    -- Check to make sure the setup window isn't active already
    if (self.setupWindowIsActive) then
        return;
    else
        self.setupWindowIsActive = true;
    end

    self.UIComponents.Input = {
        SpecificItemRules = {},
    };

    -- Divide the PackMule rules in 3 types:
    --      1 Rule that decides where item lower than quality X get sent
    --      1 Rule that decides where item higher than quality X get sent
    --      Multiple rules that decide where specific items go (based on name)
    local LowerThanRule;
    local HigherThanRule;
    local SpecificItemRules = {};
    for _, Rule in pairs(Settings:get("PackMule.Rules", {})) do
        if (Rule.item) then
            tinsert(SpecificItemRules, Rule);
        elseif (Rule.operator) then
            if (Rule.operator == "<") then
                LowerThanRule = Rule;
            elseif (Rule.operator == ">") then
                HigherThanRule = Rule;
            end
        end
    end

    -- Create a container/parent frame
    local Window = AceGUI:Create("Frame");
    Window:SetCallback("OnClose", function(widget)
        self:processSettings();
        AceGUI:Release(widget);
        self.setupWindowIsActive = false;
    end);
    Window:SetTitle("Gargul v" .. GL.version);
    Window:SetStatusText("Addon v" .. GL.version);
    Window:SetLayout("Flow");
    Window:SetWidth(600);
    Window:SetHeight(450);
    Window:EnableResize(false);
    Window.statustext:GetParent():Hide(); -- Hide the statustext bar
    Window:EnableResize(false);

    -- Make sure the window can be closed by pressing the escape button
    _G["GARGUL_PACKMULE_WINDOW"] = Window.frame;
    tinsert(UISpecialFrames, "GARGUL_PACKMULE_WINDOW");

    self:drawSpacer(Window, 1, 10);

    --[[ INFO ]]
    self:drawInfoSection(Window);

    --[[ SETTINGS ]]
    self:drawHeader(Window, "Settings");
    self:drawSettingsSection(Window);
    self:drawSpacer(Window, 1, 10);

    --[[ ITEM QUALITY RULES ]]
    self:drawHeader(Window, "Item Quality");
    self:drawLowerThanQualityRule(Window, LowerThanRule);
    self:drawSpacer(Window, 1, 5);
    self:drawHigherThanQualityRule(Window, HigherThanRule);
    self:drawSpacer(Window, 1, 20);

    --[[ SPECIFIC ITEM RULES ]]
    self:drawHeader(Window, "Specific Items");

    local ScrollFrame = self:drawScrollFrame(Window);

    for _, Rule in pairs(SpecificItemRules) do
        self:drawSpecifItemRule(ScrollFrame, Rule);
    end

    -- Make sure to draw additional item rules in case we have less than 4
    for index = #self.UIComponents.Input.SpecificItemRules, 4 do
        self:drawSpecifItemRule(ScrollFrame);
    end
end

function PackMuleUI:drawInfoSection(Frame)
    local Row = AceGUI:Create("SimpleGroup");
    Row:SetLayout("Flow");
    Row:SetFullWidth(true);
    Row:SetHeight(30);
    Frame:AddChild(Row);

    self:drawSpacer(Row, 25, 1);

    -- LABEL: Info about PackMule
    local Info = AceGUI:Create("Label");
    Info:SetText("Only works if you are loot master! Use SELF to refer to yourself, IGNORE to ignore an item or RANDOM to send items to random players! PackMule ignores recipes, quest items and a list of untradable items (see 'Show Ignored Items' button below!) |cffC41E3ANB: Player names are CASE SENSITIVE!|r");
    Info:SetHeight(20);
    Info:SetWidth(520);
    Info:SetJustifyH("CENTER");
    Row:AddChild(Info);
end

function PackMuleUI:drawSettingsSection(Frame)
    GL:debug("PackMuleUI:drawSettingsSection");

    local Row = AceGUI:Create("SimpleGroup");
    Row:SetLayout("Flow");
    Row:SetFullWidth(true);
    Row:SetHeight(30);
    Frame:AddChild(Row);

    self:drawSpacer(Row, 35, 1);

    local EnablePackMule = AceGUI:Create("CheckBox");
    EnablePackMule:SetLabel("Enable");
    EnablePackMule:SetValue(Settings:get("PackMule.enabled", false));
    EnablePackMule:SetWidth(120);
    EnablePackMule:SetCallback("OnEnter", function()
        GameTooltip:SetOwner(EnablePackMule.frame, "ANCHOR_TOP");
        GameTooltip:SetText("The rules will only be applied if this box is checked");
        GameTooltip:Show();
    end);
    EnablePackMule:SetCallback("OnLeave", function()
        GameTooltip:Hide();
    end);
    Row:AddChild(EnablePackMule);
    self.UIComponents.Input.Enabled = EnablePackMule;

    local PackMulePersistsAfterZoneChange = AceGUI:Create("CheckBox");
    PackMulePersistsAfterZoneChange:SetLabel("Persist after zone switch");
    PackMulePersistsAfterZoneChange:SetValue(Settings:get("PackMule.persistsAfterZoneChange", false));
    PackMulePersistsAfterZoneChange:SetCallback("OnEnter", function()
        GameTooltip:SetOwner(PackMulePersistsAfterZoneChange.frame, "ANCHOR_TOP");
        GameTooltip:SetText([[
PackMule is disabled by default after you as a player change to a different zone (e.g. you enter or leave an instance).
If you check this box then PackMule will keep its former state even after moving between zones. |cff8aecffUse with caution!
]]);
        GameTooltip:Show();
    end);
    PackMulePersistsAfterZoneChange:SetCallback("OnLeave", function()
        GameTooltip:Hide();
    end);
    Row:AddChild(PackMulePersistsAfterZoneChange);
    self.UIComponents.Input.persistAfterZoneChange = PackMulePersistsAfterZoneChange;

    local PackMulePersistsAfterReload = AceGUI:Create("CheckBox");
    PackMulePersistsAfterReload:SetLabel("Persist after reload");
    PackMulePersistsAfterReload:SetValue(Settings:get("PackMule.persistsAfterReload", false));
    PackMulePersistsAfterReload:SetCallback("OnEnter", function()
        GameTooltip:SetOwner(PackMulePersistsAfterReload.frame, "ANCHOR_TOP");
        GameTooltip:SetText([[
PackMule is disabled by default after you as a player reload (e.g. reload the UI or restart the game).
If you check this box then PackMule will keep its former state even after reloading. |c00be3333Use with EXTREME caution|r!
]]);
        GameTooltip:Show();
    end);
    PackMulePersistsAfterReload:SetCallback("OnLeave", function()
        GameTooltip:Hide();
    end);
    Row:AddChild(PackMulePersistsAfterReload);
    self.UIComponents.Input.persistAfterReload = PackMulePersistsAfterReload;
end

function PackMuleUI:drawScrollFrame(Frame)
    local ScrollFrameParent = AceGUI:Create("SimpleGroup");
    ScrollFrameParent:SetLayout("Fill");
    ScrollFrameParent:SetFullWidth(true);
    ScrollFrameParent:SetHeight(110);
    Frame:AddChild(ScrollFrameParent);

    local ScrollFrame = AceGUI:Create("ScrollFrame");
    ScrollFrame:SetLayout("Flow");
    ScrollFrameParent:AddChild(ScrollFrame);

    local AddRuleButton = AceGUI:Create("Button");
    AddRuleButton:SetText("Add Item Rule");
    AddRuleButton:SetWidth(140);
    AddRuleButton:SetHeight(20);
    AddRuleButton:SetCallback("OnClick", function()
        self:drawSpecifItemRule(ScrollFrame);
    end);

    self:drawSpacer(Frame, 1, 10);

    local Row = AceGUI:Create("SimpleGroup");
    Row:SetLayout("Flow");
    Row:SetFullWidth(true);
    Row:SetHeight(30);
    Frame:AddChild(Row);

    self:drawSpacer(Row, 35, 1);
    Row:AddChild(AddRuleButton);

    self:drawSpacer(Row, 50, 10);

    local IgnoredItems = GL.AceGUI:Create("Button");
    IgnoredItems:SetText("Show Ignored Items");
    IgnoredItems:SetHeight(20);
    IgnoredItems:SetCallback("OnClick", function()
        GL.Interface.PackMule.IgnoredItems:draw();
    end);
    Row:AddChild(IgnoredItems);

    return ScrollFrame;
end

function PackMuleUI:drawSpecifItemRule(Frame, Rule)
    GL:debug("PackMuleUI:drawSpecifItemRule");

    Rule = Rule or {
        item = "",
        target = "",
    };

    local Row = AceGUI:Create("SimpleGroup");
    Row:SetLayout("Flow");
    Row:SetFullWidth(true);
    Row:SetHeight(30);
    Frame:AddChild(Row);

    self:drawSpacer(Row, 35, 1);

    -- LABEL: Send loot of quality lower than
    local beforeItemLabel = AceGUI:Create("Label");
    local ruleIndex = #self.UIComponents.Input.SpecificItemRules + 1;
    beforeItemLabel:SetText(ruleIndex .. ". Send item Name or ID");
    beforeItemLabel:SetHeight(20);
    beforeItemLabel:SetWidth(140);
    Row:AddChild(beforeItemLabel);

    -- ITEM NAME
    local ItemName = AceGUI:Create("EditBox");
    ItemName:DisableButton(true);
    ItemName:SetHeight(25);
    ItemName:SetWidth(150);
    ItemName:SetText(Rule.item);
    Row:AddChild(ItemName);

    -- Increase space between input and next label ("to")
    self:drawSpacer(Row, 8, 1);

    -- LABEL: to
    local afterItemLabel = AceGUI:Create("Label");
    afterItemLabel:SetText("to");
    afterItemLabel:SetHeight(20);
    afterItemLabel:SetWidth(16);
    Row:AddChild(afterItemLabel);

    local ItemTarget = AceGUI:Create("EditBox");
    ItemTarget:DisableButton(true);
    ItemTarget:SetHeight(25);
    ItemTarget:SetWidth(150);
    ItemTarget:SetText(Rule.target);
    Row:AddChild(ItemTarget);

    tinsert(self.UIComponents.Input.SpecificItemRules, {
        Name = ItemName,
        Target = ItemTarget
    });
end

function PackMuleUI:drawSpacer(Parent, width, height)
    GL:debug("self:drawSpacer");

    local Spacer = AceGUI:Create("SimpleGroup");
    Spacer:SetLayout("Flow");
    Spacer:SetWidth(width);
    Spacer:SetHeight(height);
    Parent:AddChild(Spacer);
end

function PackMuleUI:drawLowerThanQualityRule(Frame, Rule)
    GL:debug("PackMuleUI:drawLowerThanQualityRule");

    Rule = Rule or {
        quality = nil,
        target = "",
    }

    local Row = AceGUI:Create("SimpleGroup");
    Row:SetLayout("Flow");
    Row:SetFullWidth(true);
    Row:SetHeight(30);
    Frame:AddChild(Row);

    self:drawSpacer(Row, 35, 1);

    -- LABEL: Send loot of quality lower than
    local beforeLowerThanRuleLabel = AceGUI:Create("Label");
    beforeLowerThanRuleLabel:SetText("Send loot of quality |c00f7922elower|r than");
    beforeLowerThanRuleLabel:SetHeight(20);
    beforeLowerThanRuleLabel:SetWidth(170);
    Row:AddChild(beforeLowerThanRuleLabel);

    -- DROPDOWN
    local LowerThanRuleQuality = AceGUI:Create("Dropdown");
    LowerThanRuleQuality:SetHeight(20);
    LowerThanRuleQuality:SetWidth(120);
    LowerThanRuleQuality:SetList({
        [0] = "0 - Poor",
        [1] = "1 - Common",
        [2] = "2 - Uncommon",
        [3] = "3 - Rare",
        [4] = "4 - Epic",
        [5] = "5 - Legendary"
    });
    LowerThanRuleQuality:SetValue(Rule.quality);
    Row:AddChild(LowerThanRuleQuality);
    self.UIComponents.Input.LowerThanRuleQuality = LowerThanRuleQuality;

    -- Increase space between dropdown and next label ("to")
    self:drawSpacer(Row, 8, 1);

    -- LABEL: to
    local afterLowerThanRuleLabel = AceGUI:Create("Label");
    afterLowerThanRuleLabel:SetText("to");
    afterLowerThanRuleLabel:SetHeight(20);
    afterLowerThanRuleLabel:SetWidth(16);
    Row:AddChild(afterLowerThanRuleLabel);

    local LowerThanRuleTarget = AceGUI:Create("EditBox");
    LowerThanRuleTarget:DisableButton(true);
    LowerThanRuleTarget:SetHeight(25);
    LowerThanRuleTarget:SetWidth(150);
    LowerThanRuleTarget:SetText(Rule.target);
    Row:AddChild(LowerThanRuleTarget);
    self.UIComponents.Input.LowerThanRuleTarget = LowerThanRuleTarget;
end

function PackMuleUI:drawHigherThanQualityRule(Frame, Rule)
    GL:debug("PackMuleUI:drawHigherThanQualityRule");

    Rule = Rule or {
        quality = nil,
        target = "",
    }

    local Row = AceGUI:Create("SimpleGroup");
    Row:SetLayout("Flow");
    Row:SetFullWidth(true);
    Row:SetHeight(40);
    Frame:AddChild(Row);

    self:drawSpacer(Row, 35, 1);

    -- LABEL: Send loot of quality lower than
    local beforeHigherThanRuleLabel = AceGUI:Create("Label");
    beforeHigherThanRuleLabel:SetText("Send loot of quality |c00f7922ehigher|r than");
    beforeHigherThanRuleLabel:SetHeight(20);
    beforeHigherThanRuleLabel:SetWidth(170);
    Row:AddChild(beforeHigherThanRuleLabel);

    -- DROPDOWN
    local HigherThanRuleQuality = AceGUI:Create("Dropdown");
    HigherThanRuleQuality:SetHeight(20);
    HigherThanRuleQuality:SetWidth(120);
    HigherThanRuleQuality:SetList({
        [0] = "0 - Poor",
        [1] = "1 - Common",
        [2] = "2 - Uncommon",
        [3] = "3 - Rare",
        [4] = "4 - Epic",
        [5] = "5 - Legendary"
    });
    HigherThanRuleQuality:SetValue(Rule.quality);
    Row:AddChild(HigherThanRuleQuality);
    self.UIComponents.Input.HigherThanRuleQuality = HigherThanRuleQuality;

    -- Increase space between dropdown and next label ("to")
    self:drawSpacer(Row, 8, 1);

    -- LABEL: to
    local afterHigherThanRuleLabel = AceGUI:Create("Label");
    afterHigherThanRuleLabel:SetText("to");
    afterHigherThanRuleLabel:SetHeight(20);
    afterHigherThanRuleLabel:SetWidth(16);
    Row:AddChild(afterHigherThanRuleLabel);

    local HigherThanRuleTarget = AceGUI:Create("EditBox");
    HigherThanRuleTarget:DisableButton(true);
    HigherThanRuleTarget:SetHeight(25);
    HigherThanRuleTarget:SetWidth(150);
    HigherThanRuleTarget:SetText(Rule.target);
    Row:AddChild(HigherThanRuleTarget);
    self.UIComponents.Input.HigherThanRuleTarget = HigherThanRuleTarget;
end

function PackMuleUI:drawHeader(Frame, text)
    GL:debug("PackMuleUI:drawHeader");

    local Heading = AceGUI:Create("Heading");
    Heading:SetFullWidth(true);
    Heading:SetText(text);
    Frame:AddChild(Heading);
end

-- Process all values of the PackMule settings window
function PackMuleUI:processSettings()
    GL:debug("PackMuleUI:processSettings");

    local enabled = self.UIComponents.Input.Enabled:GetValue();
    local persistAfterReload = self.UIComponents.Input.persistAfterReload:GetValue();
    local persistAfterZoneChange = self.UIComponents.Input.persistAfterZoneChange:GetValue();

    -- Lower/higher than quality rules
    local lowerThanRuleQuality = self.UIComponents.Input.LowerThanRuleQuality:GetValue();
    local lowerThanRuleTarget = tostring(self.UIComponents.Input.LowerThanRuleTarget:GetText());
    local higherThanRuleQuality = self.UIComponents.Input.HigherThanRuleQuality:GetValue();
    local higherThanRuleTarget = tostring(self.UIComponents.Input.HigherThanRuleTarget:GetText());

    -- Reset (clear) all the current PackMule rules
    GL.PackMule:resetRules();

    Settings:set("PackMule.enabled", enabled);
    Settings:set("PackMule.persistsAfterZoneChange", persistAfterZoneChange);
    Settings:set("PackMule.persistsAfterReload", persistAfterReload);

    -- Add the item-specific rules first
    for _, SpecificItemRule in pairs(self.UIComponents.Input.SpecificItemRules) do
        local name = tostring(SpecificItemRule.Name:GetText());
        local target = tostring(SpecificItemRule.Target:GetText());

        GL.PackMule:addRule({
            item = name,
            target = target,
        });
    end

    -- Add the quality lower than X rule
    if (type(lowerThanRuleQuality) ~= "nil"
            and lowerThanRuleTarget
            and lowerThanRuleTarget ~= ""
    ) then
        GL.PackMule:addRule({
            quality = lowerThanRuleQuality,
            operator = "<",
            target = lowerThanRuleTarget,
        });
    end

    -- Add the quality higher than X rule
    if (type(higherThanRuleQuality) ~= "nil"
            and higherThanRuleTarget
            and higherThanRuleTarget ~= ""
    ) then
        GL.PackMule:addRule({
            quality = higherThanRuleQuality,
            operator = ">",
            target = higherThanRuleTarget,
        });
    end
end

GL:debug("PackMuleUI.lua");